"""
SMF - Enterprise MCP Framework

A production-ready framework built on FastMCP for creating,
structuring, and deploying MCP servers.
"""

from smf.core import AppBuilder, ServerFactory, create_server
from smf.registry import ComponentRegistry, ComponentMetadata
from smf.settings import Settings, get_settings, set_settings

__version__ = "0.1.0"

__all__ = [
    "AppBuilder",
    "ServerFactory",
    "create_server",
    "ComponentRegistry",
    "ComponentMetadata",
    "Settings",
    "get_settings",
    "set_settings",
]

